c ------ Subroutine CH_XtoY (idsym,idtrf,atrf,btrf) --------------------
c Change variable from x to y(x) and transform derivatives
c
c     Author: Thorwald Herbert, OSU, 1988
c
c     idsym <  0 interval -1, 1, polynomials Tk, k=1, ...   jx+nx
c           >= 0 interval  0, 1, polynomials Tk, k=1, ... 2*jx+nx+1
c     idtrf =  0 no transformation, y=x
c              1 linear transformation (a,b) to (-1,1): y=(a+b+(b-a)x)/2
c              2 transformation (0,inf) to (1,0): y=a/x-a
c              3 transformation (0,inf) to (1,0): y=-a*log(x)
c              4 transformation (-inf,inf) to (-1,1): y=a*x/sqrt(1-x**2)
c              5 transformation (-inf,inf) to (-1,1): y=a*atanh(x)
c     atrf  :  parameter of the transformation or unused
c     btrf  :  parameter of the transformation or unused
c -----------------------------------------------------------------------

      subroutine chxtoy(idsym,idtrf,atrf,btrf)
#include "common.ins"
      common /collot/ chx(0:jx),chtx(0:jx,0:2*jx+nx+1,0:nx)
     |                         ,chty(0:jx,0:2*jx+nx+1,0:nx)
      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)

c provide for symmetry

      if (idsym.lt.0) then
         kx=jx+nx
      else
         kx=2*jx+nx+1
      endif

c choose transformation

      go to (1,10,20,20,20,20),idtrf+1
      print*, 'CH_XtoY: Illegal transformation'
      stop

c idtrf=0, y=x

1     do 2 j=0,jx
         chy(j)=chx(j)
         do 2 n=0,nx
            do 2 k=0,kx
               chty(j,k,n)=chtx(j,k,n)
2     continue
      return

c idtrf=1, y=(a+b+(b-a)x)/2

10    x0=(atrf+btrf)/2
      x1=2/(btrf-atrf)
      do 11 j=0,jx
         chy(j)=x0+chx(j)/x1
         do 11 n=0,nx
            xn=x1**n
            do 11 k=0,kx
               chty(j,k,n)=chtx(j,k,n)*xn
11    continue
      return

c idtrf=2, y=a/x-a
c idtrf=3, y=-a*log(x)
c idtrf=4, y=a*x/sqrt(1-x**2)
c idtrf=5, y=a*atanh(x)

20    nn=nx
      do 21 j=0,jx
         go to (22,23,24,25),idtrf-1

22       chy(j)=atrf/chx(j)-atrf
         x0=chx(j)/atrf
         x1=-chx(j)*x0
         x2=-2*x1*x0
         x3=-3*x2*x0
         x4=-4*x3*x0
         go to 30

23       chy(j)=-atrf*log(chx(j))
         x0=1/atrf
         x1=-chx(j)*x0
         x2=-x1*x0
         x3=-x2*x0
         x4=-x3*x0
         go to 30

24       chy(j)=atrf*chx(j)/sqrt(1-chx(j)**2)
         x0=1/sqrt(chy(j)**2+atrf**2)
         x1=(1-chx(j)**2)*x0
         x2=-3*chx(j)*x1*x0
         x3=(15*chx(j)**2-3)*x1*x0**2
         x4=chx(j)*(45-105*chx(j)**2)*x1*x0**3
         go to 30

25       chy(j)=atrf*log((1+chx(j))/(1-chx(j)))/2
         x0=1/atrf
         x1=(1-chx(j)**2)*x0
         x2=-2*chx(j)*x1*x0
         x3=2*(2*chx(j)**2*x0-x1)*x1*x0
         x4=8*chx(j)*(2*x1-chx(j)**2*x0)*x0**2

30       continue
         do 31 k=0,kx
            chty(j,k,0)=chtx(j,k,0)
31       continue
         if (nn.lt.1) go to 21
         do 32 k=0,kx
            chty(j,k,1)=chtx(j,k,1)*x1
32       continue
         if (nn.lt.2) go to 21
         do 33 k=0,kx
            chty(j,k,2)=chtx(j,k,2)*x1**2+chtx(j,k,1)*x2
33       continue
         if (nn.lt.3) go to 21
         do 34 k=0,kx
            chty(j,k,3)=(chtx(j,k,3)*x1**2+3*chtx(j,k,2)*x2)*x1
     |                  +chtx(j,k,1)*x3
34       continue
         if (nn.lt.4) go to 21
         do 35 k=0,kx
            chty(j,k,4)=(chtx(j,k,4)*x1**2+6*chtx(j,k,3)*x2)*x1**2
     |                  +chtx(j,k,2)*(3*x2**2+4*x1*x3)+chtx(j,k,1)*x4
35       continue
         if (nn.lt.5) go to 21
         print*, 'CH_XtoY: Dont know how to get higher derivatives'
         stop
21    continue
      end
