c ------ Subroutine H_LOCAL --------------------------------------------
c Set the matrices A and B of the homogeneous system and solve the
c homogeneous system (A-sigma*B) a = 0 for a single eigenvalue
c
c     Author: Thorwald Herbert, OSU, 1988
c
c ----------------------------------------------------------------------

      subroutine hlocal(idev,indt,iter)
c **********************************************************************
c Perform 64-bit arithmetic
      implicit real*8 (a-h,o-z)

c Number of collocation points jx
      PARAMETER (JX= 30)

c Number of equations and variables
      PARAMETER (NEQU= 1)
      parameter (nvar=nequ,isize=(jx+1)*nequ)

c Highest derivative nx
      PARAMETER (NX= 4)

c Number of constants (or zero)
      PARAMETER (NCON= 0)

c Number of parameters
      PARAMETER (NPAR= 6)
c **********************************************************************
c Numbers of input/output units
      parameter (iodefs=99)
      parameter (iopars=98)
      parameter (iologi=97)
      parameter (iologo=96)
      parameter (iospec=95)
      parameter (iofunc=94)
      parameter (ioseri=93)
      parameter (iotabl=92)
      parameter (iocurv=91)
      PARAMETER (IOSTAT=80)
c **********************************************************************
c Arrays for basic state
      COMMON /MSTATE/ DISPL,UM(0:JX,0:2),RVM(0:JX,0:2)
c **********************************************************************

      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)
      common /params/ con(0:ncon),par(npar),dpar(npar,2),ipar(npar,2),
     |                itask
      common /vector/ pvec(0:jx,1:nequ,0:nx,1:nvar,0:npar),
     |                rvec(0:jx,1:nequ,0:nx,1:nvar,2)
      common /locmat/ amat(0:jx,1:nequ,0:jx,1:nvar),
     |                arhs(0:jx,1:nequ)
      common /coeffs/ coef(0:jx,1:nvar)
      dimension itmp(isize)
      complex*16 pvec,rvec,uimag,zero,amat,arhs,coef,sigma,samat,sprev,
     |           dsigma,zz
      data itmax/30/,itnew/8/,epsilon/1e-7/

      zro=0
      one=1
      uimag=cmplx(zro,one)
      zero=cmplx(zro,zro)

c obtain coefficient matrices at collocation points for given parameters

      itcur=0
8     call pvector

c set the matrix A

      do 1 j=0,jx
        do 1 i=1,nequ
          do 1 k=0,jx
            do 1 l=1,nvar
              amat(j,i,k,l)=zero
              do 1 n=0,nx
                amat(j,i,k,l)=amat(j,i,k,l)
     |                       +chfy(j,k,n,l)*pvec(j,i,n,l,0)

1     continue

      do 2 m=1,npar
      if (dpar(m,1).ne.0) sigma=cmplx(par(m),par(m+1))
2     continue
      samat=sigma
      sprev=sigma

c reduce system, find first eigenvector if idev < 0

      call algc(amat,itmp,isize,isize,0,idev,dsigma)
      if (idev.lt.0) then
         idev=1
         do 3 j=0,jx
            do 3 i=1,nequ
               coef(j,i)=arhs(j,i)
3        continue
      endif

c wielandt step

9     itcur=itcur+1
      call qvector
      do 4 j=0,jx
        do 4 i=1,nequ
          arhs(j,i)=zero
          do 4 k=0,jx
            do 4 l=1,nvar
              zz=zero
              do 5 n=0,nx
                zz=zz+chfy(j,k,n,l)*rvec(j,i,n,l,1)
5             continue
          arhs(j,i)=arhs(j,i)-zz*coef(k,l)
4     continue
      call algc(amat,itmp,isize,isize,0,0,dsigma)
      sigma=samat+dsigma
      do 6 m=1,npar
      if (dpar(m,1).ne.0) then
         par(m)=sigma
         par(m+1)=-uimag*sigma
      endif
6     continue
      do 7 j=0,jx
         do 7 i=1,nequ
            coef(j,i)=arhs(j,i)
7     continue

c check for convergence

      zz=sigma-sprev
      error=abs(zz)
      if (abs(sigma).gt.1) error=error/abs(sigma)
      iter=itcur
      if (error.ge.epsilon) then
         sprev=sigma
         if (itcur.le.itmax) then
            if (indt.gt.0) then
               if (indt.gt.1) call bstate
               go to 8
            else if (mod(itcur,itnew).eq.0) then
               go to 8
            else
               go to 9
            endif
         else
            iter=-itcur
            print 2000,error
2000        format ('H_LOCAL: error=',1pe11.4)
            write(iologo,2000)error
         endif
      endif
      return
      end
