C*ALGR* SOLVE A REAL ALGEBRAIC SYSTEM  **********************
C
C       Author: Thorwald Herbert, VPI & SU
C
C       Q = matrix, dimensioned for maximum NX rows
C       IQ= integer array of maximum dimension NX
C       NX= maximum dimension of matrix
C       N = actual dimension of matrix, N.le.NX
C       NR= number of right hand sides 
C       ID= 1: reduce, solve for NR right hand sides
C       ID= 0, NR > 0: solve for NR new right hand sides
C       ID= 0, NR = 0: Wielandt correction step
C       ID=-1: reduce, first Wielandt step
C       E = Wielandt correction
C
      SUBROUTINE ALGR(Q,IQ,NX,N,NR,ID,E)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION Q(NX,1),IQ(1)
C
      M=N+NR
      M1=N-1
      N1=N+1
      IF (ID.EQ.0) GO TO 10
      DO 1 K=1,M1
      XX=0
      KX=K
      DO 2 KK=K,N
      X=ABS(Q(KK,K))
      IF (X.LE.XX) GO TO 2
      XX=X
      KX=KK
2     CONTINUE
      IQ(K)=KX
      IF (KX.EQ.K) GO TO 3
      DO 4 JJ=K,M
      Z=Q(K,JJ)
      Q(K,JJ)=Q(KX,JJ)
4     Q(KX,JJ)=Z
3     CONTINUE
      K1=K+1
      DO 1 KK=K1,N
      IF (ABS(Q(KK,K)).LE.0) GO TO 1
      Q(KK,K)=Q(KK,K)/Q(K,K)
      DO 5 JJ=K1,M
5     Q(KK,JJ)=Q(KK,JJ)-Q(KK,K)*Q(K,JJ)
1     CONTINUE
      IF (ID.GT.0) GO TO 12
C
      Q(N,N1)=1
      DO 6 KK=1,M1
      K=N-KK
      Q(K,N1)=-Q(K,N)
      IF (K.EQ.M1) GO TO 6
      K1=K+1
      DO 7 L=K1,M1
7     Q(K,N1)=Q(K,N1)-Q(K,L)*Q(L,N1)
6     Q(K,N1)=Q(K,N1)/Q(K,K)
      RETURN
C
10    IF (NR.LE.0) M=N1
      DO 11 L=N1,M
      DO 11 K=1,M1
      KX=IQ(K)
      Z=Q(K,L)
      Q(K,L)=Q(KX,L)
      Q(KX,L)=Z
      K1=K+1
      DO 11 KK=K1,N
11    Q(KK,L)=Q(KK,L)-Q(KK,K)*Q(K,L)
      IF (NR.GT.0) GO TO 12
C
      E=Q(N,N)/Q(N,M)
      DO 13 K=1,N
13    Q(K,M)=E*Q(K,M)
C
12    IF (M.LE.N) RETURN
      DO 14 KK=1,N
      K=N1-KK
      IF (K.EQ.N) GO TO 15
      K1=K+1
      DO 16 L=N1,M
      DO 16 LL=K1,N
16    Q(K,L)=Q(K,L)-Q(K,LL)*Q(LL,L)
15    DO 14 L=N1,M
14    Q(K,L)=Q(K,L)/Q(K,K)
      RETURN
      END
