C **********************************************************************
C
C  SUBROUTINE ZGEFA
C
C **********************************************************************
      SUBROUTINE ZGEFA(A,LDA,N,IPVT,INFO)
      INTEGER LDA,N,IPVT(N),INFO

c **********************************************************************
c Perform 64-bit arithmetic
#ifdef CRAY
#define ZERO 0.0E0
#define ONE 1.0E0
#define D_ABS ABS
#define D_REAL REAL
#define D_IMAG AIMAG
      REAL D_ABS, D_REAL, D_IMAG
#define C_MPLEX COMPLEX
#else
#define ZERO 0.0D0
#define ONE 1.0D0
#define D_ABS DABS
#define D_REAL DREAL
#define D_IMAG DIMAG
      DOUBLE PRECISION D_ABS, D_REAL, D_IMAG
#define C_MPLEX COMPLEX*16
#endif
c **********************************************************************

      C_MPLEX A(LDA,N)
C
C     ZGEFA FACTORS A COMPLEX MATRIX BY GAUSSIAN ELIMINATION.
C
C     ZGEFA IS USUALLY CALLED BY ZGECO, BUT IT CAN BE CALLED
C     DIRECTLY WITH A SAVING IN TIME IF  RCOND  IS NOT NEEDED.
C     (TIME FOR ZGECO) = (1 + 9/N)*(TIME FOR ZGEFA) .
C
C     ON ENTRY
C
C        A       C_MPLEX(LDA, N)
C                THE MATRIX TO BE FACTORED.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY  A .
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX  A .
C
C     ON RETURN
C
C        A       AN UPPER TRIANGULAR MATRIX AND THE MULTIPLIERS
C                WHICH WERE USED TO OBTAIN IT.
C                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
C                L  IS A PRODUCT OF PERMUTATION AND UNIT LOWER
C                TRIANGULAR MATRICES AND  U  IS UPPER TRIANGULAR.
C
C        IPVT    INTEGER(N)
C                AN INTEGER VECTOR OF PIVOT INDICES.
C
C        INFO    INTEGER
C                = 0  NORMAL VALUE.
C                = K  IF  U(K,K) .EQ. 0.0 .  THIS IS NOT AN ERROR
C                     CONDITION FOR THIS SUBROUTINE, BUT IT DOES
C                     INDICATE THAT ZGESL OR ZGEDI WILL DIVIDE BY ZERO
C                     IF CALLED.  USE  RCOND  IN ZGECO FOR A RELIABLE
C                     INDICATION OF SINGULARITY.
C
C     LINPACK. THIS VERSION DATED 08/14/78 .
C     CLEVE MOLER, UNIVERSITY OF NEW MEXICO, ARGONNE NATIONAL LAB.
C
C     SUBROUTINES AND FUNCTIONS
C
C     BLAS ZAXPY,ZSCAL,IZAMAX
C     FORTRAN D_ABS
C
C     INTERNAL VARIABLES
C
      C_MPLEX T
      INTEGER IZAMAX,J,K,KP1,L,NM1
C
      C_MPLEX ZDUM
C     C_MPLEX ZDUMR,ZDUMI
C     D_REAL(ZDUMR) = ZDUMR
C     D_IMAG(ZDUMI) = (ZERO,-ONE)*ZDUMI
      D_CABS1(ZDUM) = D_ABS(D_REAL(ZDUM)) + D_ABS(D_IMAG(ZDUM))
C
C     GAUSSIAN ELIMINATION WITH PARTIAL PIVOTING
C
      INFO = 0
      NM1 = N - 1
      IF (NM1 .LT. 1) GO TO 70
      DO 60 K = 1, NM1
         KP1 = K + 1
C
C        FIND L = PIVOT INDEX
C
         L = IZAMAX(N-K+1,A(K,K),1) + K - 1
         IPVT(K) = L
C
C        ZERO PIVOT IMPLIES THIS COLUMN ALREADY TRIANGULARIZED
C
         IF (D_CABS1(A(L,K)) .EQ. ZERO) GO TO 40
C
C           INTERCHANGE IF NECESSARY
C
            IF (L .EQ. K) GO TO 10
               T = A(L,K)
               A(L,K) = A(K,K)
               A(K,K) = T
   10       CONTINUE
C
C           COMPUTE MULTIPLIERS
C
            T = -(ONE,ZERO)/A(K,K)
            CALL ZSCAL(N-K,T,A(K+1,K),1)
C
C           ROW ELIMINATION WITH COLUMN INDEXING
C
            DO 30 J = KP1, N
               T = A(L,J)
               IF (L .EQ. K) GO TO 20
                  A(L,J) = A(K,J)
                  A(K,J) = T
   20          CONTINUE
               CALL ZAXPY(N-K,T,A(K+1,K),1,A(K+1,J),1)
   30       CONTINUE
         GO TO 50
   40    CONTINUE
            INFO = K
   50    CONTINUE
   60 CONTINUE
   70 CONTINUE
      IPVT(N) = N
      IF (D_CABS1(A(N,N)) .EQ. ZERO) INFO = N
      RETURN
      END
