c ------ Subroutine H_LOCALR -------------------------------------------
c Set the matrices A and B of the homogeneous system and solve the
c homogeneous system (A-B(sigma1,sigma2)) a = 0 for two real values
c sigma1, sigma2 which are not real and imaginary part of a complex
c eigenvalue (method of augmented matrix)
c
c     Author: Thorwald Herbert, OSU, 1988
c
c ----------------------------------------------------------------------

      subroutine hlocalr(idev,indt,iter)
#include "common.ins"
      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)
      common /params/ con(0:ncon),par(npar),dpar(npar),ipar(npar),
     |                itask
      common /vector/ pvec(0:jx,1:nequ,0:nx,1:nvar,0:npar),
     |                rvec(0:jx,1:nequ,0:nx,1:nvar)
      common /lormat/ cmat(isize+1,isize+1),arhs(isize+1)
      common /coeffs/ coef(0:jx,1:nvar)
      dimension itmp(2*isize+2),sig(2)
      equivalence (amat,cmat)
      data itmax/20/,epsilon/1e-7/

c find first eigenvector if idev < 0

      if (idev.lt.0) then
        do 1 j=0,jx
          do 1 i=1,nequ
            jj=(i-1)*jx+i+j
            do 1 k=0,jx
              do 1 l=1,nvar
                kk=(l-1)*jx+l+k
                cmat(jj,1,kk)=0
                do 2 n=0,nx
                  cmat(jj,1,kk)=cmat(jj,1,kk)
     |                         +chfy(j,k,n,l)*pvec(j,i,n,l,0)
2               continue
1       continue
        do 3 jj=1,isize
           cmat(isize,1,jj)=0
           cmat(isize,2,jj)=0
           cmat(jj,1,isize+1)=0
3       continue
        cmat(isize,isize)=1
        amat(1,isize,1,isize+1)=1
        call algr(amat,itmp,2*isize+2,2*isize,1,1,zz)
        do 4 j=0,jx
          do 4 i=1,nequ
            jj=(i-1)*jx+i+j
            coef(j,i)=cmat(jj,1,isize+1)
4       continue
        idev=1
      endif

c obtain coefficient matrices at collocation points for given parameters

      itcur=0
12    call pvector

c set the matrix A

      do 5 j=0,jx
        do 5 i=1,nequ
          jj=(i-1)*jx+i+j
          do 5 k=0,jx
            do 5 l=1,nvar
              kk=(l-1)*jx+l+k
              cmat(jj,1,kk)=0
              do 6 n=0,nx
                cmat(jj,1,kk)=cmat(jj,1,kk)
     |                       +chfy(j,k,n,l)*pvec(j,i,n,l,0)
6             continue
              amat(1,jj,2,kk)=-amat(2,jj,1,kk)
              amat(2,jj,2,kk)=amat(1,jj,1,kk)
5     continue

c augment the matrix

      itcur=itcur+1
      call qvector
      do 8 j=0,jx
        do 8 i=1,nequ
          jj=(i-1)*jx+i+j
          cmat(isize+1,1,jj)=0
          cmat(isize+1,2,jj)=0
          cmat(jj,1,isize+1)=0
          cmat(jj,2,isize+1)=0
          cmat(jj,1,isize+2)=0
          do 8 k=0,jx
            do 8 l=1,nvar
              s(1)=0
              s(2)=0
              do 9 n=0,nx
                s(1)=s(1)-chfy(j,k,n,l)*rvec(j,i,n,l,1)
                s(2)=s(2)-chfy(j,k,n,l)*rvec(j,i,n,l,2)
9             continue
          cmat(jj,1,isize+1)=cmat(jj,1,isize+1)+s(1)*coef(k,l)
          cmat(jj,2,isize+1)=cmat(jj,2,isize+1)+s(2)*coef(k,l)
8     continue
      amat(1,isize+1,1,isize)=1
      amat(2,isize+1,2,isize)=1
      cmat(isize+1,1,isize+1)=0
      cmat(isize+1,2,isize+1)=0
      amat(1,isize+1,1,isize+2)=1
      amat(2,isize+1,1,isize+2)=0

      call algr(amat,itmp,2*isize+2,2*isize+2,1,1,zz)

c new solution

      error=0
      do 10 m=1,npar
         x=par(m)
         par(m)=par(m)-arhs(1,isize+1)*dpar(m,1)
         par(m)=par(m)-arhs(2,isize+1)*dpar(m,2)
         errm=abs(par(m)-x)
         if (abs(par(m)).gt.1) errm=errm/abs(par(m))
         error=max(error,errm)
10    continue
      do 11 j=0,jx
        do 11 i=1,nequ
          jj=(i-1)*jx+i+j
          coef(j,i)=cmat(jj,1,isize+2)
11    continue

c check for convergence

      iter=itcur
      if (error.ge.epsilon) then
         samat=sigma
         if (itcur.le.itmax) then
            if (indt.gt.1) call bstate
            go to 12
         else
            iter=-itcur
            print 2000,error
2000        format ('H_LOCALR: error=',1pe11.4)
            write(iologo,2000)error
         endif
      endif
      return
      end
