c ------ Subroutine H_VALUES -------------------------------------------
c Print variables in spectral and physical space (F and T coefficients)
c
c     Author: Thorwald Herbert, OSU, 1988
c
c ----------------------------------------------------------------------

      subroutine hvalues
c **********************************************************************
c Perform 64-bit arithmetic
      implicit real*8 (a-h,o-z)

c Number of collocation points jx
      PARAMETER (JX= 8)

c Number of equations and variables
      PARAMETER (NEQU= 2)
      parameter (nvar=nequ,isize=(jx+1)*nequ)

c Highest derivative nx
      PARAMETER (NX= 4)

c Number of constants (or zero)
      PARAMETER (NCON= 0)

c Number of parameters
      PARAMETER (NPAR= 4)
c **********************************************************************
c Numbers of input/output units
      parameter (iodefs=99)
      parameter (iopars=98)
      parameter (iologi=97)
      parameter (iologo=96)
      parameter (iospec=95)
      parameter (iofunc=94)
      parameter (ioseri=93)
      parameter (iotabl=92)
      parameter (iocurv=91)
c **********************************************************************
c Arrays for basic state
      COMMON /MSTATE/ GM(0:JX),GMDERIV(0:JX)
c **********************************************************************
      common /collot/ chx(0:jx),chtx(0:jx,0:2*jx+nx+1,0:nx)
     |                         ,chty(0:jx,0:2*jx+nx+1,0:nx)
      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)
      common /contrl/ idsy(nvar),idbc(nvar),idpts,idtrf,atrf,btrf
      common /coeffs/ coef(0:jx,1:nvar)
      common /coeffn/ coefn(0:jx,1:nvar),coetn(0:jx+nx,1:nvar),
     |                jxt(1:nvar)
      common /params/ con(0:ncon),par(npar),dpar(npar),ipar(npar),
     |                itask
      common /names/  conname(0:ncon),parname(npar),varname(nvar),
     |                problem
      common /funct/  phys(0:jx,0:nx,1:nvar)
      character*8 conname,parname,varname
      character*80 problem

c find norm: sum coefn^2 = 1

      div=0
      do 1 n=1,nvar
         do 1 k=0,jx
         div=div+coef(k,n)**2
1     continue
      div=sqrt(div)

c provide for symmetry

      do 2 n=1,nvar
      print*, 'Series for ',varname(n),'  (l,T[l],k,F[k])'
      write(ioseri,*) 'Series for ',varname(n),'  (l,T[l],k,F[k])'
      write(iologo,*) 'Series for ',varname(n),'  (l,T[l],k,F[k])'
      if (idsy(n).lt.0) then
         k0=0
         k1=1
      else if (idsy(n).eq.0) then
         k0=0
         k1=2
      else
         k0=1
         k1=2
      endif

c transform coefficients in F into coefficients in T

        do 3 k=0,jx+nx
3       coetn(k,n)=0
        do 4 k=0,jx
          coefn(k,n)=coef(k,n)/div
          l=k0+k*k1
          go to (10,11,12,13),idbc(n)+1

c for definitions of F see subroutine chttof.f
c no boundary condition

10        coetn(k,n)=coefn(k,n)
          go to 5

c boundary condition 1: f(a)=f(b)=0

11        coetn(k     ,n)=coetn(k     ,n)+coefn(k,n)
          coetn(k+2/k1,n)=coetn(k+2/k1,n)-coefn(k,n)
          go to 5

c boundary condition 2: f'(a)=f'(b)=0

12        coetn(k     ,n)=coetn(k     ,n)+coefn(k,n)*(l+1)**2/(l+2)**2
          if (l.gt.0)
     |       coetn(k+2/k1,n)=coetn(k+2/k1,n)-coefn(k,n)*(l+1)**2/l**2
          go to 5

c boundary condition 3: f(a)=f(b)=f'(a)=f'(b)=0

13        coetn(k     ,n)=coetn(k     ,n)+coefn(k,n)*(2*l+4)/(l+3)
          coetn(k+2/k1,n)=coetn(k+2/k1,n)-coefn(k,n)
          coetn(k+4/k1,n)=coetn(k+4/k1,n)+coefn(k,n)*(2*l+4)/(l+1)

5         print 2000,l,coetn(k,n),k,coefn(k,n)
2000      format(2(i4,1pe16.7))
          write(ioseri,2000)l,coetn(k,n),k,coefn(k,n)
          write(iologo,2000)l,coetn(k,n),k,coefn(k,n)
4       continue 
        jxt(n)=jx
        if (idbc(n).gt.0) then
          print 2000,l+k1,coetn(jx+1,n)
          write(ioseri,2000)l+k1,coetn(jx+1,n)
          write(iologo,2000)l+k1,coetn(jx+1,n)
          jxt(n)=jx+1
        endif
        if (idbc(n).gt.2) then
          print 2000,l+2*k1,coetn(jx+2,n)
          write(ioseri,2000)l+2*k1,coetn(jx+2,n)
          write(iologo,2000)l+2*k1,coetn(jx+2,n)
          jxt(n)=jx+2
        endif
      print*
      write(iologo,*)
2     continue

c function values at collocation points

      do 6 l=1,nvar
      print*, 'Values of ',varname(l),' (j,x,y,value)'
      write(iofunc,*) 'Values of ',varname(l),' (j,x,y,value)'
      write(iologo,*) 'Values of ',varname(l),' (j,x,y,value)'
        do 7 j=0,jx
          do 8 n=0,nx
            phys(j,n,l)=0
            do 8 k=0,jx
              phys(j,n,l)=phys(j,n,l)+coefn(k,l)*chfy(j,k,n,l)
8         continue
          print'(i4,1p4e15.7)/(34x,2e15.7)',
     |    j,chx(j),chy(j),(phys(j,n,l),n=0,0)
          write(iofunc,'(i4,1p4e15.7)/(34x,2e15.7)')
     |    j,chx(j),chy(j),(phys(j,n,l),n=0,0)
          write(iologo,'(i4,1p4e15.7)/(34x,2e15.7)')
     |    j,chx(j),chy(j),(phys(j,n,l),n=0,0)
7       continue
      if (l.eq.nvar) go to 6
      print*
      write(iologo,*)
6     continue
      close(iofunc)
      return
      end
