c ------ Subroutine P_VECTOR -------------------------------------------
c Set the coefficient matrix P at collocation points for given sigma
c
c     Author: Thorwald Herbert, OSU, 1988
c
c ----------------------------------------------------------------------

      subroutine pvector
c **********************************************************************
c Perform 64-bit arithmetic
      implicit real*8 (a-h,o-z)

c Number of collocation points jx
      PARAMETER (JX= 8)

c Number of equations and variables
      PARAMETER (NEQU= 2)
      parameter (nvar=nequ,isize=(jx+1)*nequ)

c Highest derivative nx
      PARAMETER (NX= 4)

c Number of constants (or zero)
      PARAMETER (NCON= 0)

c Number of parameters
      PARAMETER (NPAR= 4)
c **********************************************************************
c Numbers of input/output units
      parameter (iodefs=99)
      parameter (iopars=98)
      parameter (iologi=97)
      parameter (iologo=96)
      parameter (iospec=95)
      parameter (iofunc=94)
      parameter (ioseri=93)
      parameter (iotabl=92)
      parameter (iocurv=91)
c **********************************************************************
c Arrays for basic state
      COMMON /MSTATE/ GM(0:JX),GMDERIV(0:JX)
c **********************************************************************
      common /collof/ chy(0:jx),chfy(0:jx,0:jx,0:nx,1:nvar)
      common /vector/ pvec(0:jx,1:nequ,0:nx,1:nvar,0:npar),
     |                rvec(0:jx,1:nequ,0:nx,1:nvar)
      common /params/ con(0:ncon),par(npar),dpar(npar),ipar(npar),
     |                itask
      common /names/  conname(0:ncon),parname(npar),varname(nvar),
     |                problem
      character*8 conname,parname,varname
      character*80 problem

c reset coefficient matrices to zero

      do 1 j=0,jx
        do 1 i=1,nequ
           do 1 n=0,nx
              do 1 l=1,nvar
                 pvec(j,i,n,l,0)=0
1     continue

c set coefficient matrix at collocation points for given parameters

      ideriv=0
      do 2 j=0,jx

c **********************************************************************
c **********************************************************************
c pvec(j,i,n,l,m)  with j = collocation point (set by program)
c                       i = i-th equation
c                       n = n-th derivative of
c                       l = l-th variable
c                       m = derivative wrto m-th parameter

c CASE: Circular Couette flow in a small gap (assume real sigma)
c
c i=1: (D^2-alpha^2-sigma)(D^2-alpha^2) u     + Ta alpha^2 g(y) v = 0
c i=2:                                - u + (D^2-alpha^2-sigma) v = 0
c                                      l=1                     l=2
      OMEGA=PAR(1)
      TA   =PAR(2)
      ALPHA=PAR(3) 
      SIGMA=PAR(4)

      if (ideriv.eq.0) then

         PVEC(J,1,4,1,0)=1
         PVEC(J,1,2,1,0)=-2*ALPHA**2-SIGMA
         PVEC(J,1,0,1,0)=ALPHA**2*(ALPHA**2+SIGMA)

         PVEC(J,1,0,2,0)=TA*ALPHA**2*GM(J)

         PVEC(J,2,0,1,0)=-1

         PVEC(J,2,2,2,0)=1
         PVEC(J,2,0,2,0)=-ALPHA**2-SIGMA

      else

c derivative d / d(omega)

         PVEC(J,1,0,2,1)=-TA*ALPHA**2*GMDERIV(J)

c derivative d / d(Ta)

         PVEC(J,1,0,2,2)=ALPHA**2*GM(J)

c derivative d / d(alpha)

         PVEC(J,1,2,1,3)=-4*ALPHA
         PVEC(J,1,0,1,3)=2*ALPHA*(2*ALPHA**2+SIGMA)
         PVEC(J,1,0,2,3)=2*ALPHA*TA*GM(J)

         PVEC(J,2,0,2,3)=-2*ALPHA

c derivative d / d(sigma)

         PVEC(J,1,2,1,4)=-1
         PVEC(J,1,0,1,4)=ALPHA**2

         PVEC(J,2,0,2,4)=-1

      endif

c **********************************************************************

c **********************************************************************

2     continue
      return
      end
